/*  einsc -- a compiler for eins
 *    Symbol table entries.
 *
 *    Matthias Zenger, Michel Schinz, Philippe Altherr
 *    $Id: Symbol.java,v 1.1 2005/03/15 14:46:50 dragos Exp $
 */
package einsc;


/** This class provides a representation for symbol table entries.
 */
class Symbol {

    /** Kinds.
     */
    public static final int VAR   = 0; // block variable or function parameter
    public static final int FUNC  = 1; // function
    public static final int CLASS = 2; // class
    public static final int FIELD = 3; // class field

    /** The position of the symbol's definition.
     */
    public final int pos;

    /** The name of the symbol.
     */
    public final String name;

    /** The type of the symbol.
     */
    public Type type;

    /** The kind of the symbol.
     */
    public final int kind;

    /** The next symbol in the scope.
     */
    public Symbol next;

    public Symbol(int pos, String name, Type type, int kind) {
        this.pos = pos;
        this.name = name;
        this.type = type;
        this.kind = kind;
    }

    public String toString() {
        return name + ": " + kind + "[" + type + "]";
    }
   
    /** Offset of the symbol. For variables (VAR), that's the offset
     * relative to the frame pointer, for functions (FUNC) that's the
     * PC of the first instruction, for class fields (FIELD) that's
     * their index in the class, for classes (CLASS) that represents
     * nothing.
     *
     * This field is given a value and used by the code generator. It
     * is ignored by the analyzer.
     */
    public int offset;

}
